<?php
 $prefix    = "..";
 $logo      = "abakus.jpg";
 $logo_text = "russischer Abakus";
 include "$prefix/scripts/header.inc"
?>
<h1>Manual Page of <tt>vam</tt></h1>



<h2><a name="name">NAME</a></h2>
<p>vam.pl - Virtual Assembler Machine: an interpreter for a simple processor's instruction set</p>


<h2><a name="synopsis">SYNOPSIS</a></h2>
<p>vam.pl [--help] [--manual | -M] [--verbose ] [--Version]
       [--input infile] [--output outfile]
       [--registers count] [--memory size]
       [--dump | -D]
       [--statistics]
       [srcfile...]</p>


<h2><a name="options">OPTIONS</a></h2>
<dl>
<dt><strong><a name="item__2d_2dinput_infile"><strong>--input</strong> <em>infile</em></a></strong>
</dt>
<dd>
Read the input of the interpreted program, from <em>infile</em>.
</dd>
<dd>
<p>Default: <em>stdin</em>.</p>
</dd>

<dt><strong><a name="item__2d_2doutput_outfile"><strong>--output</strong> <em>outfile</em></a></strong>
</dt>
<dd>
Write the output of the interpreted program, to <em>outfile</em>.
</dd>
<dd>
<p>Default: <em>stdout</em>.</p>
</dd>

<dt><strong><a name="item__2d_2dmemory_size"><strong>--memory</strong> <em>size</em></a></strong>
</dt>
<dd>
Size of the avaliable memory.
<em>size</em> is a positive number followed by <strong>k</strong> (kilo) or
<strong>M</strong> (mega), which specifies the size in kilo or mega memory cells
respectively.
</dd>
<dd>
<p>Default: 32M</p>
</dd>

<dt><strong><a name="item__2d_2dregisters_count"><strong>--registers</strong> <em>count</em></a></strong>
</dt>
<dd>
Number of the available registers: <strong>R0</strong>, <strong>R1</strong>, ... <strong>R</strong><em>count-1</em>.
</dd>
<dd>
<p>Default: 32</p>
</dd>

<dt><strong><a name="item__2d_2dhelp"><strong>--help</strong></a></strong>
</dt>
<dd>
Show this help and terminate.
</dd>

<dt><strong><a name="item__2d_2dmanual__2dm"><strong>--manual</strong> <strong>-M</strong></a></strong>
</dt>
<dd>
Show the entire manual and terminate.
</dd>

<dt><strong><a name="item__2d_2dverbose"><strong>--verbose</strong></a></strong>
</dt>
<dd>
Verbose.
</dd>

<dt><strong><a name="item__2d_2dversion"><strong>--Version</strong></a></strong>
</dt>
<dd>
Show program version.
</dd>

<dt><strong><a name="item__2d_2ddump"><strong>--dump</strong></a></strong>
</dt>
<dt><strong><a name="item__2dd"><strong>-D</strong></a></strong>
</dt>
<dd>
After parsing the source file, dump the instruction list.
</dd>

<dt><strong><a name="item__2d_2dstatistics"><strong>--statistics</strong></a></strong>
</dt>
<dd>
Emit some statistical information about the program.
</dd>
</dl>
<p>Option names may abbreviated as long as the are unique.
Instead of --XX -XX may be used.</p>


<h2><a name="description">DESCRIPTION</a></h2>
<p><strong>vam.pl</strong> is an interpreter of a simple processor's (designed by me and called
<strong>VAM</strong>, Virtual Assembler Processor) instruction set. It has the ``usual''
machine instructions available on ``real'' processors.</p>
<p><strong>vam.pl</strong> may be used e.g. in a compiler construction course to have a ``play
ground'' when generating machine code.</p>
<p><strong>vam.pl</strong> read the instructions to be interpreted either from <em>stdin</em> or
the <code>file(s)</code> <em>srcfile</em> given on the command line.
The <em>BIOS</em> (Basic Input / Output System) of the processor allows
to read and write a file during runtime (see <strong>--input</strong> and <strong>--output</strong>).</p>


<h2><a name="format_of_the_source_file">FORMAT OF THE SOURCE FILE</a></h2>
<p>The source file <em>srcfile</em> contains the assembler instructions to be
interpreted.</p>
<p>A line is either empty, contains a comment or <em>one</em> instruction or <em>one</em>
(string or data) declaration.</p>
<p>The # character starts a comment up to the end of the line.</p>
<p>The instruction and registers names are case insensitive.
Labels and string literals are case sensitive.</p>


<h2><a name="architecture_of_the_processor_vam">ARCHITECTURE OF THE PROCESSOR VAM</a></h2>
<p>The VAM processor has a usual <em>von-Neumann</em> architecture with registers,
memory, an ALU (arithmetic logical unit) etc.</p>
<p>The processor knows two data types: (signed) integers and floating point numbers.
Memory addresses are positive integer values.</p>
<p>The number of registers may be specified by the <strong>--registers</strong> command line
option and are denoted by <strong>R0</strong>, <strong>R1</strong>, ... A register may hold <em>one</em> value of
a given data type at a time.</p>
<p>The memory is partitioned into <em>memory cells</em>, which may hold <em>one</em> value of
a given data type at a time.</p>

<h3><a name="special_registers">Special Registers</a></h3>
<p>The registers <strong>R0</strong>, <strong>R1</strong> und <strong>R2</strong> are reserved and should not be used
for other purposes (even it is legal to modify them).
All other registers may be used without any restrictions.</p>
<dl>
<dt><strong><a name="item_r0"><strong>R0</strong></a></strong>
</dt>
<dd>
The register <strong>R0</strong> is used as <em>instruction counter</em> and set to 1 before
the program starts.
</dd>
<dd>
<p>With the exception of the control flow and non-executable statements, after
performing the operation specified by an instruction, the instruction counter
<strong>R0</strong> will be incremented by 1.</p>
</dd>

<dt><strong><a name="item_r1"><strong>R1</strong></a></strong>
</dt>
<dd>
The register <strong>R1</strong> is used as stack pointer pointing to the top of the stack
maintained by some instructions (see below). At program start <strong>R1</strong> is initialized to
the value of <em>high-memory</em>.
</dd>

<dt><strong><a name="item_r2"><strong>R2</strong></a></strong>
</dt>
<dd>
The register <strong>R2</strong> holds the pointer to the first unused memory cell.  The the
free memory may be used manage dynamically allocated memory similar to the
<em>sbrk</em>(2) call of the C-library.
</dd>
</dl>

<h3><a name="layout_of_the_memory">Layout of the Memory</a></h3>
<pre>
    Address        | Description
   ----------------+-------------------------------------------------
       0           | Undefined address
   ----------------+-------------------------------------------------
       1           | The instruction to be executed at program start
      ...          | More instructions.
     &lt;code-end&gt;    | The textual last instruction
   ----------------+-------------------------------------------------
    &lt;data-start&gt;   | Global data
     ...           | ........
    &lt;data-end&gt;     | ........
   ----------------+-------------------------------------------------
    &lt;malloc-start&gt; | Start of free memory.
     ...           | ....
    &lt;high-memory&gt;-1| Last memory cell, initial top of stack
   ----------------+-------------------------------------------------</pre>
<p>The free memory ``grows'' towards larger address, while the stack grows towards
small addresses.</p>

<h3><a name="vam_runtime_errors">VAM Runtime Errors</a></h3>
<p>The VAM emits under the following conditions an error message and terminates:</p>
<dl>
<dt></dt>
<dd>
* VAM does not emit an error if the two areas overlap.
</dd>

<dt></dt>
<dd>
* VAM emits an error if the memory at address 0 or an address &gt;= <em>high-memory</em>
is accessed.
</dd>

<dt></dt>
<dd>
* VAM emits an error, if a memory cell not containing an instruction should be
read as instruction and executed.
</dd>

<dt></dt>
<dd>
* Division by 0 causes an error.
</dd>
</dl>


<h2><a name="machine_instructions">MACHINE INSTRUCTIONS</a></h2>
<p>In the sequel the instruction are declared, which the processor is able to
process.
In the description below <em>r</em>, <em>r1</em>, <em>r2</em>, <em>r3</em> specify any processor
register <strong>R0</strong>, <strong>R1</strong>, ..., <strong>R</strong><em>max_registers-1</em>, even the reserved ones.
If for an instruction several registers may be specified, the same register may
be used several times.</p>
<p>The notion <strong>M[</strong><em>address</em><strong>]</strong> specifies the access of the memory cell
at the given <em>address</em>.</p>

<h3><a name="labels">Labels</a></h3>
<p>There are three kinds of named entities, named by so called <em>label</em>:</p>
<ol>
<li><strong><a name="item_data_declaration_3a">Data declaration:</a></strong>
</li>
<em>label</em> <strong>:</strong> <strong>DATA</strong> &lt;count&gt;

<li><strong><a name="item_string_declaration">String declaration</a></strong>
</li>
<em>label</em> <strong>:</strong> <strong>``</strong>character string<strong>''</strong>

<li><strong><a name="item_instruction">Instruction</a></strong>
</li>
<em>label</em> <strong>:</strong> <em>one-instruction</em>
</ol>
<p>A <em>label</em> consists of a letter followed by letters, digits or an underscore
character. The labels of each entity class (data, string, instruction)
reside in a separate namespace and must be unique in that namespace.</p>
<p>The string and data declarations are non-executable instructions, therefore the
instruction counter is not modified. They may occur everywhere in the source
file.</p>
<p>Memory cells holding global data must be reserved and a label must be given be
given.  This declaration reserves <em>count</em> memory cells. The address of the
first memory cell may be accessed using the name <em>label</em>.</p>
<p>A string declaration declares a name for a character string used in the
<strong>write_s</strong> instruction.  The two characters ``\n'' represent a newline. The two
characters ``\t'' represent a tabulator.</p>
<p>A control flow statement may use an instruction <em>label</em> to specify the next
statement to be executed.</p>
<p>Some instructions have a suffix in their name.  The suffix <strong>_f</strong> indicates
floating point and <strong>_i</strong> integer operation. The suffix <strong>_c</strong> indicates that the
instruction takes a literal integer argument, <strong>_r</strong> a register is
used and <strong>_l</strong> a label.</p>
<p>A prefix <strong>i</strong> indicates some kind of indirection.</p>

<h3><a name="control_flow_instructions">Control Flow Instructions</a></h3>
<dl>
<dt><strong><a name="item_nop"><strong>nop</strong></a></strong>
</dt>
<dd>
No Operation, do nothing and continue execution with the next instruction.
If a label declaration is not followed by an instruction,
a <strong>nop</strong> is assumed.
</dd>

<dt><strong><a name="item_end"><strong>end</strong></a></strong>
</dt>
<dd>
Stop the processor and terminate the interpretation.
</dd>

<dt><strong><a name="item_goto_label"><strong>goto</strong> <em>label</em></a></strong>
</dt>
<dt><strong><a name="item_igoto_r"><strong>igoto</strong> <em>r</em></a></strong>
</dt>
<dd>
Continue execution with the instruction named by <em>label</em>, or at the address
stored in register <em>r</em>.
</dd>

<dt><strong><a name="item_call_r1__2c_label"><strong>call</strong>  <em>r1</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dt><strong><a name="item_icall_r1__2c_r2"><strong>icall</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dd>
Continue execution with the instruction named by <em>label</em> (or the address
stored in register <em>r2</em>) and push the address of the statement following the
<strong>call</strong> onto the stack pointed to by <em>r2</em>.
</dd>
<dd>
<p><em>r1</em> = <em>r1</em> - 1; Memory[<em>r1</em>] = <strong>R0</strong> + 1; <strong>R0</strong> = <em>label</em></p>
</dd>
<dd>
<p><em>r1</em> = <em>r1</em> - 1; Memory[<em>r1</em>] = <strong>R0</strong> + 1; <strong>R0</strong> = <em>r2</em></p>
</dd>

<dt><strong><a name="item_return_r"><strong>return</strong> <em>r</em></a></strong>
</dt>
<dd>
Get address from the top of the stack <em>r</em>, pop it and continue execution
at the instruction specified by that address.
</dd>
<dd>
<p><strong>R0</strong> = Memory[<em>r</em>]; <em>r</em> = <em>r</em> + 1;</p>
</dd>
</dl>

<h3><a name="conditional_control_flow_instructions">Conditional Control Flow Instructions</a></h3>
<p>If register <em>r</em> holds the shown condition, then continue execution with the
instruction specified by <em>label</em>, else continue with the textual next
instruction.</p>
<p>The registers should contain an integer value.</p>
<p>See also <strong>cmp_i</strong> and <strong>cmp_f</strong>.</p>
<dl>
<dt><strong><a name="item_iflt_r__2c_label"><strong>iflt</strong>   <em>r</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dd>
less then:     <em>r</em> &lt;  0
</dd>

<dt><strong><a name="item_ifle_r__2c_label"><strong>ifle</strong>   <em>r</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dd>
less equal:    <em>r</em> &lt;= 0
</dd>

<dt><strong><a name="item_ifeq_r__2c_label"><strong>ifeq</strong>   <em>r</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dd>
equal:         <em>r</em> =  0
</dd>

<dt><strong><a name="item_ifne_r__2c_label"><strong>ifne</strong>   <em>r</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dd>
not equal:     <em>r</em> != 0
</dd>

<dt><strong><a name="item_ifgt_r__2c_label"><strong>ifgt</strong>   <em>r</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dd>
greater then:  <em>r</em> &gt;  0
</dd>

<dt><strong><a name="item_ifge_r__2c_label"><strong>ifge</strong>   <em>r</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dd>
greater equal: <em>r</em> &gt;= 0
</dd>

<dt><strong><a name="item_iftrue_r__2c_label"><strong>iftrue</strong> <em>r</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dd>
true-value:    <em>r</em> != 0
</dd>

<dt><strong><strong>iftrue</strong> <em>r</em> <strong>,</strong> <em>label</em></strong>
</dt>
<dd>
false-value:   <em>r</em> == 0
</dd>
</dl>

<h3><a name="arithmetic_instructions">Arithmetic instructions</a></h3>
<p>Except when noted, all registers used by an instruction must hold values of
the same datatype, which must match the type of the instruction.
Otherwise the result may be undefined.</p>
<p>The processing of an instructions read the source registers performs the
operation and store the result in the destination register.</p>
<p>There is no test over underflow or overflow of the result. A division by 0
causes an error.</p>
<dl>
<dt><strong><a name="item_add_i_r1__2c_r2__2c_r3"><strong>add_i</strong>  <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item_add_f_r1__2c_r2__2c_r3"><strong>add_f</strong>  <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Addition: <em>r1</em> =  <em>r2</em> +  <em>r3</em>
</dd>

<dt><strong><a name="item_sub_i_r1__2c_r2__2c_r3"><strong>sub_i</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item_sub_f_r1__2c_r2__2c_r3"><strong>sub_f</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Subtraction: <em>r1</em> =  <em>r2</em> - <em>r3</em>
</dd>

<dt><strong><a name="item_mult_i_r1__2c_r2__2c_r3"><strong>mult_i</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item_mult_f_r1__2c_r2__2c_r3"><strong>mult_f</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Multiplication: <em>r1</em> =  <em>r2</em> * <em>r3</em>
</dd>

<dt><strong><a name="item_div_i_r1__2c_r2__2c_r3"><strong>div_i</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item_div_f_r1__2c_r2__2c_r3"><strong>div_f</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Division: <em>r1</em> =  <em>r2</em> / <em>r3</em>
</dd>
<dd>
<p>If <em>r3</em> == 0 an error message is emitted and the program is terminated.</p>
</dd>

<dt><strong><a name="item_mod_i_r1__2c_r2__2c_r3"><strong>mod_i</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Modulo: <em>r1</em> =  <em>r2</em> % <em>r3</em>
</dd>
<dd>
<p>If <em>r3</em> == 0 an error message is emitted and the program is terminated.</p>
</dd>
<dd>
<p>Note: there is no floating point mod operation!</p>
</dd>

<dt><strong><a name="item_cmp_i_r1__2c_r2__2c_r3"><strong>cmp_i</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item_cmp_f_r1__2c_r2__2c_r3"><strong>cmp_f</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
comparison: <em>r1</em> =  <em>r2</em> compare <em>r3</em>
</dd>
<dd>
<p><em>r1</em> holds an integer -1, 0, or 1 according to:</p>
</dd>
<dl>
<dt><strong><a name="item__2d1_if_r2__3c_r3">-1 if <em>r2</em> &lt;  <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item__0_if_r2__3d_3d_r3">&nbsp;0 if <em>r2</em> == <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item__1_if_r2__3e_r3">&nbsp;1 if <em>r2</em> &gt;  <em>r3</em></a></strong>
</dt>
</dl>
<dt><strong><a name="item_add_c_r1__2c_r2__2c_value"><strong>add_c</strong>  <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dt><strong><a name="item_sub_c_r1__2c_r2__2c_value"><strong>sub_c</strong>  <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dt><strong><a name="item_mult_c_r1__2c_r2__2c_value"><strong>mult_c</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dt><strong><a name="item_div_c_r1__2c_r2__2c_value"><strong>div_c</strong>  <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dt><strong><a name="item_mod_c_r1__2c_r2__2c_value"><strong>mod_c</strong>  <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dt><strong><a name="item_cmp_c_r1__2c_r2__2c_value"><strong>cmp_c</strong>  <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dd>
Integer arithmetic / comparison with an (signed) integer constant:
</dd>
<dd>
<p><em>r1</em> =  <em>r2</em> &lt;operand&gt;  <em>value</em></p>
</dd>

<dt><strong><a name="item_f2i_r1__2c_r2"><strong>f2i</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dd>
Type cast: <em>r1</em> = (int) <em>r2</em>  (in C speak)
</dd>

<dt><strong><a name="item_i2f_r1__2c_r2"><strong>i2f</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dd>
Type cast: <em>r1</em> = (float) <em>r2</em>  (in C speak)
</dd>

<dt><strong><a name="item_lshift_r1__2c_r2__2c_r3"><strong>lshift</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dt><strong><a name="item_rshift_r1__2c_r2__2c_r3"><strong>rshift</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Left shift: <em>r1</em> = <em>r2</em> &lt;&lt; <em>r3</em>  (in C speak)
</dd>
<dd>
<p>Right shift: <em>r1</em> = <em>r2</em> &gt;&gt; <em>r3</em>  (in C speak)</p>
</dd>
<dd>
<p>All registers must hold integer values.</p>
</dd>
</dl>

<h3><a name="transport_instructions">Transport Instructions</a></h3>
<p>Transport instructions move values from one place to another.</p>
<p>If a register specifies an address, it must hold an positive integer value.
Everything else is undefined.</p>
<dl>
<dt><strong><a name="item_copy_r1__2c_r2"><strong>copy</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dd>
Copies the content of registers: <em>r1</em> = <em>r2</em>
</dd>

<dt><strong><a name="item_cload_i_r__2c_value"><strong>cload_i</strong> <em>r</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dt><strong><a name="item_cload_f_r__2c_value"><strong>cload_f</strong> <em>r</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dd>
Loads the (signed) integer or floating point <em>value</em> into the register <em>r</em>.
</dd>

<dt><strong><a name="item_load_r1__2c_r2"><strong>load</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dt><strong><a name="item_load_l_r1__2c_label"><strong>load_l</strong> <em>r1</em> <strong>,</strong> <em>label</em></a></strong>
</dt>
<dt><strong><a name="item_load_c_r1__2c_r2__2c_value"><strong>load_c</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dd>
Load the content a memory cell to a register.
<em>label</em> is a data label and <em>value</em> is a signed integer.
</dd>
<dd>
<p><em>r1</em> = Memory[<em>r2</em>]</p>
</dd>
<dd>
<p><em>r1</em> = Memory[<em>label</em>]</p>
</dd>
<dd>
<p><em>r1</em> = Memory[<em>r2</em> + <em>value</em>]</p>
</dd>

<dt><strong><a name="item_iload_r1__2c_r2__2c_r3"><strong>iload</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Indirect load with register offset:
<em>r1</em> = Memory[Memory[<em>r2</em>] + <em>r3</em>]
</dd>

<dt><strong><a name="item_iload_c_r1__2c_r2__2c_value"><strong>iload_c</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dd>
Indirect load with constant offset:
<em>r1</em> = Memory[Memory[<em>r2</em>] + <em>value</em>]
</dd>
<dd>
<p><em>value</em> is an (signed) integer number.</p>
</dd>

<dt><strong><a name="item_store_r1__2c_r2"><strong>store</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dt><strong><a name="item_store_l_label__2c_r2"><strong>store_l</strong> <em>label</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dt><strong><a name="item_store_c_r1__2c_r2__2c_value"><strong>store_c</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dd>
Store the content a register in a memory cell.
<em>label</em> is a data label and <em>value</em> is a signed integer.
</dd>
<dd>
<p>Memory[<em>r1</em>] = <em>r2</em></p>
</dd>
<dd>
<p>Memory[<em>label</em>] = <em>r2</em></p>
</dd>
<dd>
<p>Memory[<em>r1</em> + <em>value</em>] = <em>r2</em></p>
</dd>

<dt><strong><a name="item_istore_r1__2c_r2__2c_r3"><strong>istore</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>r3</em></a></strong>
</dt>
<dd>
Indirect store with register offset:
Memory[Memory[<em>r1</em>] + <em>r2</em>] =  <em>r3</em>
</dd>

<dt><strong><a name="item_istore_c_r1__2c_r2__2c_value"><strong>istore_c</strong> <em>r1</em> <strong>,</strong> <em>r2</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dd>
Indirect store with constant offset:
Memory[Memory[<em>r1</em>] + <em>value</em>] = <em>r2</em>
</dd>
<dd>
<p><em>value</em> is an (signed) integer number.</p>
</dd>
</dl>

<h3><a name="stack_related_instructions">Stack Related Instructions</a></h3>
<p>Stack grows from higher to lower addresses.</p>
<dl>
<dt><strong><a name="item_push_r1__2c_r2"><strong>push</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dd>
Push the value of <em>r2</em> to the stack pointed by <em>r1</em>.
</dd>
<dd>
<p><em>r1</em> = <em>r1</em> - 1; Memory[<em>r1</em>] = <em>r2</em></p>
</dd>
<dd>
<p><em>r1</em> should contain an address, while <em>r2</em> may hold any value.</p>
</dd>

<dt><strong><a name="item_pop_r1__2c_value"><strong>pop</strong> <em>r1</em> <strong>,</strong> <em>value</em></a></strong>
</dt>
<dt><strong><a name="item_pop_r_r1__2c_r2"><strong>pop_r</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dd>
Pop <em>values</em> / <em>r2</em>  many values from the stack.
</dd>
<dd>
<p><em>r1</em> = <em>r1</em> + <em>r2</em></p>
</dd>
<dd>
<p><em>r1</em> = <em>r1</em> + <em>value</em></p>
</dd>
<dd>
<p><em>r1</em> should contain an address, while <em>r2</em> / <em>value</em> is an integer.</p>
</dd>
</dl>

<h3><a name="input_output_instructions">Input/Output Instructions</a></h3>
<p>The VAM has a build in BIOS (basic I/O system) which allows to
write character strings and numbers to a file (see <strong>--output</strong>).
Only numbers may be read from a file (see <strong>--input</strong>).</p>
<p>###############################################################################</p>

<h4><a name="input">Input</a></h4>
<p>The following instruction read some values from the input file.</p>
<dl>
<dt><strong><a name="item_read_i_r1__2c_r2"><strong>read_i</strong> <em>r1</em> <strong>,</strong> <em>r2</em></a></strong>
</dt>
<dt><strong><a name="item_read_f_r1_2c_r2"><strong>read_f</strong> <em>r1</em><strong>,</strong> <em>r2</em></a></strong>
</dt>
<dd>
Read a (signed) integer / floating point number and store it in register <em>r1</em>
and store an error code in <em>r2</em>.
If the read value is an integer or floating point constant, then <em>r2</em>
holds the value 1 (true), else 0 (false)
</dd>
<dd>
<p>If <em>r1</em> and <em>r2</em> denote the same register, no error code is stored.
In case of an error the value 0 is stored in the register.</p>
</dd>
<dd>
<p>The read number must be terminated by RETURN (newline).</p>
</dd>

<dt><strong><a name="item_eof_r">eof <em>r</em></a></strong>
</dt>
<dd>
Test if End Of File (EOF) has been reached: <em>r</em> = EOF? 1 : 0 (in C speak)
</dd>
</dl>

<h4><a name="output">Output</a></h4>
<p>The following instruction write some values to the output file.</p>
<dl>
<dt><strong><a name="item_write_s_label"><strong>write_s</strong> <em>label</em></a></strong>
</dt>
<dd>
Write the character string denoted by <em>label</em>.
</dd>

<dt><strong><a name="item_write_i_r"><strong>write_i</strong> <em>R</em></a></strong>
</dt>
<dd>
Write the integer value stored in register <em>R</em>.
</dd>

<dt><strong><a name="item_write_f_r"><strong>write_f</strong> <em>R</em></a></strong>
</dt>
<dd>
Write the floating point value stored in register <em>R</em>.
</dd>
</dl>


<h2><a name="examples">EXAMPLES</a></h2>
<p>The following program prints the square numbers 1 ... n^2:</p>
<pre>
 #############################################################
 # Description:  A VAM program computing square numbers
 #               Input:   read a number
 #               Output:  write square numbers 1 .... n^2</pre>
<pre>
 s_in:  &quot;Please input an integer: &quot;
 s_out: &quot;^2 = &quot;
 NL:    &quot;\n&quot;</pre>
<pre>
 start:
       write_s s_in
       read_i  R4, R4             # R4: n (ignore errors)
       write_s NL</pre>
<pre>
       cload_i R5, 1              # R5: i = 1</pre>
<pre>
 loop:
       cmp_i   R6, R5, R4         # if (i &gt; n) goto end
       ifgt    R6, end</pre>
<pre>
       mult_i  R6, R5, R5         # i * i</pre>
<pre>
       write_i R5                 # printf (&quot;%d^2 = %d\n&quot;, i, i*i);
       write_s s_out
       write_i R6
       write_s NL</pre>
<pre>
       add_c   R5, R5, 1          # i = i + 1</pre>
<pre>
       goto loop                  # and loop</pre>
<pre>
 end:
       end</pre>
<pre>
 #############################################################</pre>
<p>More examples will be found in the example files shipped together with this
program.</p>


<h2><a name="requirements">REQUIREMENTS</a></h2>
<p><strong>perl</strong>(1)</p>


<h2><a name="author">AUTHOR</a></h2>
<p>Dr. Jrgen Vollmer &lt;<a href="mailto:Juergen.Vollmer@informatik-vollmer.de">Juergen.Vollmer@informatik-vollmer.de</a>&gt;</p>
<p>Copyright (C) 2005 Dr. Jrgen Vollmer, Karlsruhe.</p>
<p>Homepage of <strong>VAM</strong>: <a href="http://www.informatik-vollmer.de/software/vam.html">http://www.informatik-vollmer.de/software/vam.html</a></p>
<p>If have written ``large programs'' doing some interesting job, it would be nice to
send me your VAM source.</p>
<p>If you find this software useful, I would be glad to receive a postcard from
you, showing the place where you're living:</p>
<p>Dr. Jrgen Vollmer, Viktoriastrasse 15,
D-76133 Karlsruhe, Germany.</p>


<h2><a name="license">LICENSE</a></h2>
<p>This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.</p>
<p>This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.</p>
<p>You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.</p>
http://www.gnu.org/copyleft/gpl.html

<h2><a name="version">VERSION</a></h2>
<p>Version 1.0 of 2005/02/24</p>


<h2><a name="changelog">CHANGELOG</a></h2>
<pre>
  $Log: vam.pl,v $
  Revision 1.8  2005/07/12 12:06:43  vollmer
  *** empty log message ***</pre>
<pre>
  Revision 1.7  2005/07/12 11:20:57  vollmer
  *** empty log message ***</pre>
<pre>
  Revision 1.6  2005/06/19 10:41:10  vollmer
  fixed focu</pre>
<pre>
  Revision 1.5  2005/03/11 22:58:51  vollmer
  fixed name of vam_ifne</pre>
<pre>
  Revision 1.4  2005/03/11 22:51:40  vollmer
  sub cm: if a memory cell undefined during an access, assign 0 to it</pre>
<pre>
  Revision 1.3  2005/03/11 22:48:05  vollmer
  fixed sub_f</pre>
<pre>
  Revision 1.2  2005/03/11 14:11:45  vollmer
  fixed docu</pre>
<pre>
  Revision 1.1  2005/02/24 16:55:52  vollmer
  Initial revision</pre>


<?php include "$prefix/scripts/footer.inc"?>
